program ListSortDemo;

{$APPTYPE CONSOLE}

uses
  Classes,
  SortingClosure in 'SortingClosure.pas';

{$R *.RES}

procedure InitIntList(L: TList);
begin
  L.Clear;
  L.Add(Ptr(3));
  L.Add(Ptr(1));
  L.Add(Ptr(4));
  L.Add(Ptr(2));
  L.Add(Ptr(5));
end;

procedure PrintIntList(L: TList);
var
  i: Integer;
begin
  write('{');
  for i := 0 to Pred(L.Count) do write(' ', Integer(L[i]));
  writeln(' }');
end;

type
  TSorter = class
  private
    FDescending: Boolean;
    FCompareRoutine: TListSortCompare;
    function MethodCompare(Item1, Item2: Pointer): Integer;
  public
    constructor Create;
    destructor Destroy; override;
    property Descending: Boolean read FDescending write FDescending;
    property CompareRoutine: TListSortCompare read FCompareRoutine;
  end;

{ TSorter }

constructor TSorter.Create;
begin
  inherited Create;
  FCompareRoutine := MakeSortCompare(MethodCompare);
end;

destructor TSorter.Destroy;
begin
  FreeSortCompare(FCompareRoutine);
  inherited;
end;

function TSorter.MethodCompare(Item1, Item2: Pointer): Integer;
begin
  if Descending then
    Result := Integer(Item2) - Integer(Item1)
  else
    Result := Integer(Item1) - Integer(Item2);
end;

function StandaloneCompare(Item1, Item2: Pointer): Integer;
begin
  Result := Integer(Item1) - Integer(Item2);
end;

var
  L: TList;
  Sorter: TSorter;
begin
  L := TList.Create;
  try
    // First demonstrate the normal way to sort
    InitIntList(L);
    writeln('unsorted:');
    PrintIntList(L);
    L.Sort(StandaloneCompare);
    writeln('sorted with standalone function:');
    PrintIntList(L);

    // Now demonstrate sorting with an object method
    InitIntList(L);
    Sorter := TSorter.Create;
    try
      writeln('sorted ascending:');
      L.Sort(Sorter.CompareRoutine);
      PrintIntList(L);
      // change a property of the object, but use the same sorting
      // callback
      Sorter.Descending := True;
      writeln('sorted descending:');
      L.Sort(Sorter.CompareRoutine);
      PrintIntList(L);
    finally
      Sorter.Free;
    end;
  finally
    L.Free;
  end;

  readln;
end.
